// Generated by CoffeeScript 1.6.2
(function() {
  var Encryptor, KEY, METHOD, PORT, REMOTE_PORT, SERVER, args, config, configContent, configFromArgs, createServer, fs, inet, inetAton, inetNtoa, k, net, path, timeout, util, v;

  net = require("net");

  fs = require("fs");

  path = require("path");

  util = require('util');

  args = require('./args');

  inet = require('./inet');

  Encryptor = require("./encrypt").Encryptor;

  inetNtoa = function(buf) {
    return buf[0] + "." + buf[1] + "." + buf[2] + "." + buf[3];
  };

  inetAton = function(ipStr) {
    var buf, i, parts;

    parts = ipStr.split(".");
    if (parts.length !== 4) {
      return null;
    } else {
      buf = new Buffer(4);
      i = 0;
      while (i < 4) {
        buf[i] = +parts[i];
        i++;
      }
      return buf;
    }
  };

  createServer = function(serverAddr, serverPort, port, key, method, timeout) {
    var getServer, server;

    getServer = function() {
      if (serverAddr instanceof Array) {
        return serverAddr[Math.floor(Math.random() * serverAddr.length)];
      } else {
        return serverAddr;
      }
    };
    server = net.createServer(function(connection) {
      var addrLen, addrToSend, cachedPieces, encryptor, headerLength, remote, remoteAddr, remotePort, stage;

      encryptor = new Encryptor(key, method);
      stage = 0;
      headerLength = 0;
      remote = null;
      cachedPieces = [];
      addrLen = 0;
      remoteAddr = null;
      remotePort = null;
      addrToSend = "";
      connection.on("data", function(data) {
        var aServer, addrtype, buf, cmd, e, reply, tempBuf;

        if (stage === 5) {
          data = encryptor.encrypt(data);
          if (!remote.write(data)) {
            connection.pause();
          }
          return;
        }
        if (stage === 0) {
          tempBuf = new Buffer(2);
          tempBuf.write("\u0005\u0000", 0);
          connection.write(tempBuf);
          stage = 1;
          return;
        }
        if (stage === 1) {
          try {
            cmd = data[1];
            addrtype = data[3];
            if (cmd !== 1) {
              util.log("unsupported cmd: " + cmd);
              reply = new Buffer("\u0005\u0007\u0000\u0001", "binary");
              connection.end(reply);
              return;
            }
            if (addrtype === 3) {
              addrLen = data[4];
            } else if (addrtype !== 1 && addrtype !== 4) {
              util.log("unsupported addrtype: " + addrtype);
              connection.end();
              return;
            }
            addrToSend = data.slice(3, 4).toString("binary");
            if (addrtype === 1) {
              remoteAddr = inetNtoa(data.slice(4, 8));
              addrToSend += data.slice(4, 10).toString("binary");
              remotePort = data.readUInt16BE(8);
              headerLength = 10;
            } else if (addrtype === 4) {
              remoteAddr = inet.inet_ntop(data.slice(4, 20));
              addrToSend += data.slice(4, 22).toString("binary");
              remotePort = data.readUInt16BE(20);
              headerLength = 22;
            } else {
              remoteAddr = data.slice(5, 5 + addrLen).toString("binary");
              addrToSend += data.slice(4, 5 + addrLen + 2).toString("binary");
              remotePort = data.readUInt16BE(5 + addrLen);
              headerLength = 5 + addrLen + 2;
            }
            buf = new Buffer(10);
            buf.write("\u0005\u0000\u0000\u0001", 0, 4, "binary");
            buf.write("\u0000\u0000\u0000\u0000", 4, 4, "binary");
            buf.writeInt16BE(2222, 8);
            connection.write(buf);
            aServer = getServer();
            remote = net.connect(serverPort, aServer, function() {
              var addrToSendBuf, i, piece;

              util.log("connecting " + remoteAddr + ":" + remotePort);
              addrToSendBuf = new Buffer(addrToSend, "binary");
              addrToSendBuf = encryptor.encrypt(addrToSendBuf);
              remote.write(addrToSendBuf);
              i = 0;
              while (i < cachedPieces.length) {
                piece = cachedPieces[i];
                piece = encryptor.encrypt(piece);
                remote.write(piece);
                i++;
              }
              cachedPieces = null;
              return stage = 5;
            });
            remote.on("data", function(data) {
              var e;

              try {
                data = encryptor.decrypt(data);
                if (!connection.write(data)) {
                  return remote.pause();
                }
              } catch (_error) {
                e = _error;
                return remote.emit("error", e);
              }
            });
            remote.on("end", function() {
              return connection.end();
            });
            remote.on("error", function(e) {
              util.log("remote " + remoteAddr + ":" + remotePort + " error: " + e);
              if (stage === 4) {
                connection.destroy();
                return;
              }
              return connection.end();
            });
            remote.on("drain", function() {
              return connection.resume();
            });
            remote.setTimeout(timeout, function() {
              connection.end();
              return remote.destroy();
            });
            if (data.length > headerLength) {
              buf = new Buffer(data.length - headerLength);
              data.copy(buf, 0, headerLength);
              cachedPieces.push(buf);
              buf = null;
            }
            return stage = 4;
          } catch (_error) {
            e = _error;
            util.log(e);
            connection.destroy();
            if (remote) {
              return remote.destroy();
            }
          }
        } else {
          if (stage === 4) {
            return cachedPieces.push(data);
          }
        }
      });
      connection.on("end", function() {
        if (remote) {
          return remote.destroy();
        }
      });
      connection.on("error", function(e) {
        util.log("local error: " + e);
        if (remote) {
          return remote.destroy();
        }
      });
      connection.on("drain", function() {
        if (remote && stage === 5) {
          return remote.resume();
        }
      });
      return connection.setTimeout(timeout, function() {
        if (remote) {
          remote.destroy();
        }
        return connection.destroy();
      });
    });
    server.listen(port, function() {
      return util.log("server listening at port " + port);
    });
    server.on("error", function(e) {
      if (e.code === "EADDRINUSE") {
        return util.log("Address in use, aborting");
      }
    });
    return server;
  };

  if (require.main === module) {
    console.log(args.version);
    configContent = fs.readFileSync(path.resolve(__dirname, "config.json"));
    config = JSON.parse(configContent);
    configFromArgs = args.parseArgs();
    for (k in configFromArgs) {
      v = configFromArgs[k];
      config[k] = v;
    }
    SERVER = config.server;
    REMOTE_PORT = config.server_port;
    PORT = config.local_port;
    KEY = config.password;
    METHOD = config.method;
    timeout = Math.floor(config.timeout * 1000);
    createServer(SERVER, REMOTE_PORT, PORT, KEY, METHOD, timeout);
  } else {
    exports.createServer = createServer;
  }

}).call(this);
